#' ---
#' title: "Extract tissue-specific functional and non-functional mRNA isoform pairs"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

import numpy as np
import pandas as pd
import random
import string
import time
from dfply import *

# Define an empty dictionary with lists to store the tissue specific cases.
frames = {}
for label in ("Heart", "Liver", "Kidney", "AdGland", "Forebrain", "Hindbrain", "EmbFacPro", "Lintestine", "Lung", "Limb", "Midbrain", "Ntube", "Sintestine", "Ovary", "Spleen", "Stomach", "Thymus", "Reference"):
    frames[label] = []


# We read the prediction files, select and take sum of columns with predictions, filter the predictions whose sum is 18 as reference positive and those with sum 0 as reference negative and store them in list.
# We can repeat the above for all the tissues, except, this time we check for the following conditions:
# If prediction after removing a tissue is 1, but all other predictions are 0, then this is a tissue specific non functional mRNA Isoform pair.
# If prediction after removing a tissue is 0, but all other predictions are 1, then this is a tissue specific functional mRNA Isoform pair.
a=["%02d" % x for x in range(25)]
for file in a:
    start = time.time()
    Mouse = pd.read_csv("Predictions_MMU_%s.csv.gz" % file, sep=',')
    end = time.time()
    print("File %(x)s read in %(y)s seconds" % {"x" : file, "y" : (end - start)}, flush=True)
    pred_col=[col for col in Mouse if str(col).startswith('pred_')]
    Mouse >>= mutate(Sum_all = X[pred_col].sum(axis=1))
    reference_pos = Mouse >> mask(X.Sum_all == 18) >> select(X.Iso1, X.Iso2, X.prob_All)
    reference_neg = Mouse >> mask(X.Sum_all == 0) >> select(X.Iso1, X.Iso2, X.prob_All)
    reference_temp = reference_pos >> bind_rows(reference_neg, join='inner')
    frames['Reference'].append(reference_temp)
    for pred, prob, label in zip(['pred_withoutHeart','pred_withoutLiver','pred_withoutKidney','pred_withoutAdGland','pred_withoutForebrain','pred_withoutHindbrain','pred_withoutEmbFacPro','pred_withoutLintestine','pred_withoutLung','pred_withoutLimb','pred_withoutMidbrain','pred_withoutNtube','pred_withoutSintestine','pred_withoutOvary','pred_withoutSpleen','pred_withoutStomach','pred_withoutThymus'], ['prob_withoutHeart','prob_withoutLiver','prob_withoutKidney','prob_withoutAdGland','prob_withoutForebrain','prob_withoutHindbrain','prob_withoutEmbFacPro','prob_withoutLintestine','prob_withoutLung','prob_withoutLimb','prob_withoutMidbrain','prob_withoutNtube','prob_withoutSintestine','prob_withoutOvary','prob_withoutSpleen','prob_withoutStomach','prob_withoutThymus'], ['Heart', 'Liver', 'Kidney', 'AdGland', 'Forebrain', 'Hindbrain', 'EmbFacPro', 'Lintestine', 'Lung', 'Limb', 'Midbrain', 'Ntube', 'Sintestine', 'Ovary', 'Spleen', 'Stomach', 'Thymus']):
        start1 = time.time()
        filter_col=[col for col in Mouse if str(col).startswith('pred_')]
        filter_col.remove(pred)
        Case2 = Mouse >> mask(X[pred] == 0, X.pred_All == 1, X[filter_col].sum(axis=1) == 17) >> select(X.Iso1, X.Iso2, X[prob], X.prob_All) >> mutate(Case = 'Case2')
        Case3 = Mouse >> mask(X[pred] == 1, X.pred_All == 0, X[filter_col].sum(axis=1) == 0) >> select(X.Iso1, X.Iso2, X[prob], X.prob_All) >> mutate(Case = 'Case3')
        temp = Case2 >> bind_rows(Case3, join='inner')
        frames[label].append(temp)
        end1 = time.time()
        print("Tissue %(x)s calculated in %(y)s seconds" % {"x" : label, "y" : (end1 - start1)}, flush=True)
    end = time.time()
    print("File %(x)s processed in %(y)s seconds" % {"x" : file, "y" : (end - start)}, flush=True)


# In the end we write the tissue specific as well as the reference pairs to file
for label in ("Heart", "Liver", "Kidney", "AdGland", "Forebrain", "Hindbrain", "EmbFacPro", "Lintestine", "Lung", "Limb", "Midbrain", "Ntube", "Sintestine", "Ovary", "Spleen", "Stomach", "Thymus", "Reference"):
    frames[label] = pd.concat(frames[label], axis=0, ignore_index=True)
    frames[label].to_csv('MMU_%s_Cases1.csv' % label, index=False, header=False)


