#' ---
#' title: "Merge expression profile from all RNA-Seq samples"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

## After processing with STAR and StringTie, we combine the expression profile for the mRNA Isoforms from the RNA-Seq samples.

setwd("../Data/StringTie_Output")
library(tidyverse)

#Store all paths to the t_data.ctab files that contain the expression profile.
t_data  <-
  list.files(recursive = T,
             pattern = "t_data.ctab",
             full.names = T)

# Function to read the files and select only mRNA Isoform name, gene name and FPKM values.
readDatFile <- function(f) {
  dat.fl <-
    read.table(f, header = T)[, c(6, 10, 12)] ## Notice that the columns are hard-coded.
}

# Apply the function readDatFile to read all the datasets in the list of files created above.
data.files <- lapply(t_data, readDatFile)
names(data.files) <-
  sapply(t_data, function(x)
    basename(dirname(x)))

# Combine the datasets from the list to create a dataframe with mRNA Isoform names, gene names and FPKM values
all <- do.call("cbind", data.files)
FPKM <-
  all[, c(1, 2, seq(3, ncol(all), 3))] ## We need to store the mRNA Isoform and gene names only once.

# Read the gene to mRNA Isoform file which contains the mRNA Isoform id's used by StringTie
# Map the StringTie mRNA Isoform id's into NCBI mRNA Isoform id's
MGI_GeneName <-
  read.table(
    "../ref_GRCm38.p4_top_level.gff3.gz",
    header = F,
    sep = "\t",
    quote = "\"",
    stringsAsFactors = F
  )
MGI_GeneName <- MGI_GeneName %>%
  filter(MGI_GeneName$V3 == "mRNA")
MGI_GeneName <- droplevels(unique(MGI_GeneName))
MGI_GeneName <- MGI_GeneName %>%
  mutate(Trans_ID = sapply(MGI_GeneName$V9, function(x)
    sub(".*ID= *(.*?) *;.*", "\\1", x))) %>%
  mutate(Transcript_ID = sapply(MGI_GeneName$V9, function(x)
    sub(".*transcript_id= *(.*?) *", "\\1", x))) %>%
  mutate(Gene_ID = sapply(MGI_GeneName$V9, function(x)
    sub(".*gene= *(.*?) *;.*", "\\1", x))) %>%
  dplyr::select(Transcript_ID, Gene_ID, Trans_ID)

FPKM_trans <-
  left_join(MGI_GeneName, FPKM, by = c("Trans_ID" = "ENCFF565KTC.t_name"))
FPKM_trans1 <-
  FPKM_trans[!duplicated(FPKM_trans$Transcript_ID), -c(2:4)]
colnames(FPKM_trans1) <-
  sapply(colnames(FPKM_trans1),
         gsub,
         pattern = "\\.FPKM",
         replacement = "")
colnames(FPKM_trans1) <-
  sapply(colnames(FPKM_trans1),
         gsub,
         pattern = "\\.fastq",
         replacement = "")
write.table(
  FPKM_trans1,
  "../MMU_FPKM_Transcript.txt",
  quote = F,
  sep = ",",
  col.names = T,
  row.names = F
)
